@@ -54,6 +54,16 @@ typedef enum {
 	GIT_EINVALIDSPEC = -12,
 	GIT_ECONFLICT = -13,
 	GIT_ELOCKED = -14,
+	GIT_EMODIFIED       = -15,
+	GIT_EAUTH           = -16,
+	GIT_ECERTIFICATE    = -17,
+	GIT_EAPPLIED        = -18,
+	GIT_EPEEL           = -19,
+	GIT_EEOF            = -20,
+	GIT_EINVALID        = -21,
+	GIT_EUNCOMMITTED    = -22,
+	GIT_EDIRECTORY      = -23,
+	GIT_EMERGECONFLICT  = -24,
 
 	GIT_PASSTHROUGH = -30,
 	GIT_ITEROVER = -31,
@@ -67,6 +77,7 @@ typedef struct {
 typedef struct git_time {
 	git_time_t time;
 	int offset;
+	char sign;
 } git_time;
 
 typedef struct git_signature {
@@ -115,7 +126,8 @@ typedef enum {
 	GIT_CREDTYPE_DEFAULT,
     GIT_CREDTYPE_SSH_INTERACTIVE,
     GIT_CREDTYPE_USERNAME,
-	...
+    GIT_CREDTYPE_SSH_MEMORY,
+    ...
 } git_credtype_t;
 
 typedef enum git_cert_t {
@@ -131,21 +143,21 @@ typedef enum {
 
 typedef struct {
 	git_cert_t cert_type;
+} git_cert;
+
+typedef struct {
+    git_cert parent;
 	git_cert_ssh_t type;
 	unsigned char hash_md5[16];
     unsigned char hash_sha1[20];
 } git_cert_hostkey;
 
 typedef struct {
-	git_cert_t cert_type;
+    git_cert parent;
 	void *data;
 	size_t len;
 } git_cert_x509;
 
-typedef struct {
-	git_cert_t cert_type;
-} git_cert;
-
 typedef int (*git_transport_message_cb)(const char *str, int len, void *data);
 typedef int (*git_cred_acquire_cb)(
 	git_cred **cred,
@@ -197,10 +209,30 @@ struct git_remote_callbacks {
 
 typedef struct git_remote_callbacks git_remote_callbacks;
 
+typedef enum {
+	GIT_PROXY_NONE,
+	GIT_PROXY_AUTO,
+	GIT_PROXY_SPECIFIED,
+} git_proxy_t;
+
+typedef struct {
+	unsigned int version;
+	git_proxy_t type;
+	const char *url;
+	git_cred_acquire_cb credentials;
+        git_transport_certificate_check_cb certificate_check;
+	void *payload;
+} git_proxy_options;
+
+#define GIT_PROXY_OPTIONS_VERSION ...
+int git_proxy_init_options(git_proxy_options *opts, unsigned int version);
+
 typedef struct {
 	unsigned int version;
 	unsigned int pb_parallelism;
 	git_remote_callbacks callbacks;
+	git_proxy_options proxy_opts;
+	git_strarray custom_headers;
 } git_push_options;
 
 #define GIT_PUSH_OPTIONS_VERSION ...
@@ -225,6 +257,8 @@ typedef struct {
 	git_fetch_prune_t prune;
 	int update_fetchhead;
 	git_remote_autotag_option_t download_tags;
+	git_proxy_options proxy_opts;
+	git_strarray custom_headers;
 } git_fetch_options;
 
 #define GIT_FETCH_OPTIONS_VERSION ...
@@ -274,6 +308,9 @@ int git_refspec_dst_matches(const git_refspec *refspec, const char *refname);
 int git_refspec_transform(git_buf *buf, const git_refspec *spec, const char *name);
 int git_refspec_rtransform(git_buf *buf, const git_refspec *spec, const char *name);
 
+int git_cred_username_new(
+	git_cred **out,
+	const char *username);
 int git_cred_userpass_plaintext_new(
 	git_cred **out,
 	const char *username,
@@ -287,6 +324,12 @@ int git_cred_ssh_key_new(
 int git_cred_ssh_key_from_agent(
     git_cred **out,
     const char *username);
+int git_cred_ssh_key_memory_new(
+    git_cred **out,
+    const char *username,
+    const char *publickey,
+    const char *privatekey,
+    const char *passphrase);
 
 /*
  * git_diff
@@ -336,13 +379,20 @@ typedef int (*git_diff_notify_cb)(
 	const char *matched_pathspec,
 	void *payload);
 
+typedef int (*git_diff_progress_cb)(
+	const git_diff *diff_so_far,
+	const char *old_path,
+	const char *new_path,
+	void *payload);
+
 typedef struct {
 	unsigned int version;
 	uint32_t flags;
 	git_submodule_ignore_t ignore_submodules;
 	git_strarray       pathspec;
-	git_diff_notify_cb notify_cb;
-	void              *notify_payload;
+	git_diff_notify_cb   notify_cb;
+	git_diff_progress_cb progress_cb;
+	void                *payload;
 	uint32_t    context_lines;
 	uint32_t    interhunk_lines;
 	uint16_t    id_abbrev;
@@ -371,7 +421,16 @@ int git_diff_tree_to_index(git_diff **diff, git_repository *repo, git_tree *old_
  * git_checkout
  */
 
-typedef enum { ... } git_checkout_notify_t;
+typedef enum {
+	GIT_CHECKOUT_NOTIFY_NONE      = 0,
+	GIT_CHECKOUT_NOTIFY_CONFLICT  = 1,
+	GIT_CHECKOUT_NOTIFY_DIRTY     = 2,
+	GIT_CHECKOUT_NOTIFY_UPDATED   = 4,
+	GIT_CHECKOUT_NOTIFY_UNTRACKED = 8,
+	GIT_CHECKOUT_NOTIFY_IGNORED   = 16,
+
+	GIT_CHECKOUT_NOTIFY_ALL       = 0x0FFFF
+} git_checkout_notify_t;
 
 typedef int (*git_checkout_notify_cb)(
 	git_checkout_notify_t why,
@@ -478,11 +537,12 @@ typedef ... git_config;
 typedef ... git_config_iterator;
 
 typedef enum {
-	GIT_CONFIG_LEVEL_SYSTEM = 1,
-	GIT_CONFIG_LEVEL_XDG = 2,
-	GIT_CONFIG_LEVEL_GLOBAL = 3,
-	GIT_CONFIG_LEVEL_LOCAL = 4,
-	GIT_CONFIG_LEVEL_APP = 5,
+    GIT_CONFIG_LEVEL_PROGRAMDATA = 1,
+	GIT_CONFIG_LEVEL_SYSTEM = 2,
+	GIT_CONFIG_LEVEL_XDG = 3,
+	GIT_CONFIG_LEVEL_GLOBAL = 4,
+	GIT_CONFIG_LEVEL_LOCAL = 5,
+	GIT_CONFIG_LEVEL_APP = 6,
 	GIT_CONFIG_HIGHEST_LEVEL = -1,
 } git_config_level_t;
 
@@ -513,6 +573,7 @@ int git_config_delete_entry(git_config *cfg, const char *name);
 int git_config_add_file_ondisk(git_config *cfg,
 	const char *path,
 	git_config_level_t level,
+	const git_repository *repo,
 	int force);
 
 int git_config_iterator_new(git_config_iterator **out, const git_config *cfg);
@@ -593,6 +654,23 @@ int git_graph_ahead_behind(size_t *ahead, size_t *behind, git_repository *repo,
  * git_submodule
  */
 
+typedef struct git_submodule_update_options {
+	unsigned int version;
+	git_checkout_options checkout_opts;
+	git_fetch_options fetch_opts;
+	int allow_fetch;
+} git_submodule_update_options;
+
+#define GIT_SUBMODULE_UPDATE_OPTIONS_VERSION ...
+
+int git_submodule_update_init_options(
+    git_submodule_update_options *opts,
+    unsigned int version);
+int git_submodule_update(
+    git_submodule *submodule,
+    int init,
+    git_submodule_update_options *options);
+
 int git_submodule_lookup(git_submodule **out, git_repository *repo, char *path);
 void git_submodule_free(git_submodule *subm);
 int git_submodule_open(git_repository **out, git_submodule *subm);
@@ -671,22 +749,22 @@ typedef struct git_blame_options {
 	uint16_t min_match_characters;
 	git_oid newest_commit;
 	git_oid oldest_commit;
-	uint32_t min_line;
-	uint32_t max_line;
+	size_t min_line;
+	size_t max_line;
 } git_blame_options;
 
 #define GIT_BLAME_OPTIONS_VERSION ...
 
 typedef struct git_blame_hunk {
-	uint16_t lines_in_hunk;
+	size_t lines_in_hunk;
 
 	git_oid final_commit_id;
-	uint16_t final_start_line_number;
+	size_t final_start_line_number;
 	git_signature *final_signature;
 
 	git_oid orig_commit_id;
 	const char *orig_path;
-	uint16_t orig_start_line_number;
+	size_t orig_start_line_number;
 	git_signature *orig_signature;
 
 	char boundary;
@@ -695,7 +773,7 @@ typedef struct git_blame_hunk {
 int git_blame_init_options(git_blame_options *opts, unsigned int version);
 uint32_t git_blame_get_hunk_count(git_blame *blame);
 const git_blame_hunk *git_blame_get_hunk_byindex(git_blame *blame, uint32_t index);
-const git_blame_hunk *git_blame_get_hunk_byline(git_blame *blame, uint32_t lineno);
+const git_blame_hunk *git_blame_get_hunk_byline(git_blame *blame, size_t lineno);
 int git_blame_file(git_blame **out, git_repository *repo, const char *path, git_blame_options *options);
 void git_blame_free(git_blame *blame);
 
@@ -703,7 +781,12 @@ void git_blame_free(git_blame *blame);
  * Merging
  */
 
-typedef enum { ... } git_merge_tree_flag_t;
+typedef enum {
+	GIT_MERGE_FIND_RENAMES = 1,
+	GIT_MERGE_FAIL_ON_CONFLICT = 2,
+	GIT_MERGE_SKIP_REUC = 4,
+	GIT_MERGE_NO_RECURSIVE = 8,
+} git_merge_flag_t;
 
 typedef enum {
 	GIT_MERGE_FILE_FAVOR_NORMAL = 0,
@@ -714,10 +797,12 @@ typedef enum {
 
 typedef struct {
 	unsigned int version;
-	git_merge_tree_flag_t tree_flags;
+	git_merge_flag_t flags;
 	unsigned int rename_threshold;
 	unsigned int target_limit;
 	git_diff_similarity_metric *metric;
+	unsigned int recursion_limit;
+	const char *default_driver;
 	git_merge_file_favor_t file_favor;
 	unsigned int file_flags;
 } git_merge_options;
@@ -737,7 +822,12 @@ typedef enum {
 	GIT_MERGE_FILE_STYLE_MERGE = 1,
 	GIT_MERGE_FILE_STYLE_DIFF3 = 2,
 	GIT_MERGE_FILE_SIMPLIFY_ALNUM = 4,
-} git_merge_file_flags_t;
+	GIT_MERGE_FILE_IGNORE_WHITESPACE = 8,
+	GIT_MERGE_FILE_IGNORE_WHITESPACE_CHANGE = 16,
+	GIT_MERGE_FILE_IGNORE_WHITESPACE_EOL = 32,
+	GIT_MERGE_FILE_DIFF_PATIENCE = 64,
+	GIT_MERGE_FILE_DIFF_MINIMAL = 128,
+} git_merge_file_flag_t;
 
 typedef struct {
 	unsigned int version;
@@ -745,7 +835,7 @@ typedef struct {
 	const char *our_label;
 	const char *their_label;
 	git_merge_file_favor_t favor;
-	git_merge_file_flags_t flags;
+	git_merge_file_flag_t flags;
 } git_merge_file_options;
 
 #define GIT_MERGE_OPTIONS_VERSION ...
@@ -756,6 +846,97 @@ int git_merge_trees(git_index **out, git_repository *repo, const git_tree *ances
 int git_merge_file_from_index(git_merge_file_result *out, git_repository *repo, const git_index_entry *ancestor, const git_index_entry *ours, const git_index_entry *theirs, const git_merge_file_options *opts);
 void git_merge_file_result_free(git_merge_file_result *result);
 
+/*
+ * git_stash
+ */
+
+typedef int (*git_stash_cb)(
+	size_t index, const char* message, const git_oid *stash_id, void *payload);
+
+typedef enum {
+	GIT_STASH_APPLY_PROGRESS_NONE = 0,
+	GIT_STASH_APPLY_PROGRESS_LOADING_STASH = 1,
+	GIT_STASH_APPLY_PROGRESS_ANALYZE_INDEX = 2,
+	GIT_STASH_APPLY_PROGRESS_ANALYZE_MODIFIED = 3,
+	GIT_STASH_APPLY_PROGRESS_ANALYZE_UNTRACKED = 4,
+	GIT_STASH_APPLY_PROGRESS_CHECKOUT_UNTRACKED = 5,
+	GIT_STASH_APPLY_PROGRESS_CHECKOUT_MODIFIED = 6,
+	GIT_STASH_APPLY_PROGRESS_DONE = 7,
+} git_stash_apply_progress_t;
+
+typedef int (*git_stash_apply_progress_cb)(
+	git_stash_apply_progress_t progress, void *payload);
+
+typedef enum {
+	GIT_STASH_DEFAULT = 0,
+	GIT_STASH_KEEP_INDEX = 1,
+	GIT_STASH_INCLUDE_UNTRACKED = 2,
+	GIT_STASH_INCLUDE_IGNORED = 4,
+} git_stash_flags;
+
+typedef enum {
+	GIT_STASH_APPLY_DEFAULT = 0,
+	GIT_STASH_APPLY_REINSTATE_INDEX = 1,
+} git_stash_apply_flags;
+
+typedef struct git_stash_apply_options {
+	unsigned int version;
+	git_stash_apply_flags flags;
+	git_checkout_options checkout_options;
+	git_stash_apply_progress_cb progress_cb;
+	void *progress_payload;
+} git_stash_apply_options;
+
+#define GIT_STASH_APPLY_OPTIONS_VERSION ...
+
+int git_stash_save(git_oid *out, git_repository *repo, const git_signature *stasher, const char *message, uint32_t flags);
+int git_stash_apply_init_options(git_stash_apply_options *opts, unsigned int version);
+int git_stash_apply(git_repository *repo, size_t index, const git_stash_apply_options *options);
+int git_stash_foreach(git_repository *repo, git_stash_cb callback, void *payload);
+int git_stash_drop(git_repository *repo, size_t index);
+int git_stash_pop(git_repository *repo, size_t index, const git_stash_apply_options *options);
+
+/*
+ * Describe
+ */
+
+typedef enum {
+	GIT_DESCRIBE_DEFAULT,
+	GIT_DESCRIBE_TAGS,
+	GIT_DESCRIBE_ALL,
+} git_describe_strategy_t;
+
+typedef struct git_describe_options {
+	unsigned int version;
+	unsigned int max_candidates_tags;
+	unsigned int describe_strategy;
+	const char *pattern;
+	int only_follow_first_parent;
+	int show_commit_oid_as_fallback;
+} git_describe_options;
+
+#define GIT_DESCRIBE_OPTIONS_VERSION ...
+
+int git_describe_init_options(git_describe_options *opts, unsigned int version);
+
+typedef struct {
+	unsigned int version;
+	unsigned int abbreviated_size;
+	int always_use_long_format;
+	const char *dirty_suffix;
+} git_describe_format_options;
+
+#define GIT_DESCRIBE_FORMAT_OPTIONS_VERSION ...
+
+int git_describe_init_format_options(git_describe_format_options *opts, unsigned int version);
+
+typedef ... git_describe_result;
+
+int git_describe_commit(git_describe_result **result, git_object *committish, git_describe_options *opts);
+int git_describe_workdir(git_describe_result **out, git_repository *repo, git_describe_options *opts);
+int git_describe_format(git_buf *out, const git_describe_result *result, const git_describe_format_options *opts);
+void git_describe_result_free(git_describe_result *result);
+
 #define GIT_ATTR_CHECK_FILE_THEN_INDEX ...
 #define GIT_ATTR_CHECK_INDEX_THEN_FILE ...
 #define GIT_ATTR_CHECK_INDEX_ONLY      ...
@@ -770,3 +951,5 @@ typedef enum {
 
 int git_attr_get(const char **value_out, git_repository *repo, uint32_t flags, const char *path, const char *name);
 git_attr_t git_attr_value(const char *attr);
+
+int git_revert_commit(git_index **out, git_repository *repo, git_commit *revert_commit, git_commit *our_commit, unsigned int mainline, const git_merge_options *merge_options);
